@@ -15,6 +15,9 @@ enum tgtadm_op {
 	OP_BIND,
 	OP_UNBIND,
 	OP_UPDATE,
+	OP_STATS,
+	OP_START,
+	OP_STOP,
 };
 
 enum tgtadm_mode {
@@ -22,11 +25,11 @@ enum tgtadm_mode {
 	MODE_TARGET,
 	MODE_DEVICE,
 	MODE_PORTAL,
+	MODE_LLD,
 
 	MODE_SESSION,
 	MODE_CONNECTION,
 	MODE_ACCOUNT,
-	MODE_STATS,
 };
 
 enum tgtadm_account_dir {
@@ -47,6 +50,7 @@ struct tgtadm_req {
 	uint32_t device_type;
 	uint32_t ac_dir;
 	uint32_t pack;
+	uint32_t force;
 };
 
 struct tgtadm_rsp {
